@@ -32,7 +32,11 @@
 
 #if HAVE_PAM
 # ifdef __APPLE__	/* MacOS X */
-#  include <pam/pam_appl.h>
+#  if __ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__ >= 1060
+#   include <security/pam_appl.h>
+#  else
+#   include <pam/pam_appl.h>
+#  endif
 # else
 #  include <security/pam_appl.h>
 # endif
@@ -61,8 +65,10 @@ set_expiration_status(char *exp_date, struct authen_data *data)
 	return;
     }
 
-    /* Check the expiration date, if any. If NULL, this check will return
-     * PW_OK */
+    /*
+     * Check the expiration date, if any. If NULL, this check will return
+     * PW_OK
+     */
     expired = check_expiration(exp_date);
 
     switch (expired) {
@@ -72,7 +78,7 @@ set_expiration_status(char *exp_date, struct authen_data *data)
 		   exp_date ? exp_date : "<no expiry date set>");
 
 	data->status = TAC_PLUS_AUTHEN_STATUS_PASS;
-	return;
+	break;
 
     case PW_EXPIRING:
 	if (debug & DEBUG_PASSWD_FLAG)
@@ -82,7 +88,7 @@ set_expiration_status(char *exp_date, struct authen_data *data)
 	    free(data->server_msg);
 	data->server_msg = tac_strdup("Password will expire soon");
 	data->status = TAC_PLUS_AUTHEN_STATUS_PASS;
-	return;
+	break;
 
     case PW_EXPIRED:
 	if (debug & DEBUG_PASSWD_FLAG)
@@ -92,16 +98,15 @@ set_expiration_status(char *exp_date, struct authen_data *data)
 	    free(data->server_msg);
 	data->server_msg = tac_strdup("Password has expired");
 	data->status = TAC_PLUS_AUTHEN_STATUS_FAIL;
-	return;
+	break;
 
     default:
 	report(LOG_ERR, "%s: Bogus return value %d from check_expiration",
 	       session.peer, expired);
 	data->status = TAC_PLUS_AUTHEN_STATUS_ERROR;
-	return;
+	break;
     }
 
-    /*NOTREACHED*/
     return;
 }
 
@@ -126,7 +131,7 @@ verify(char *name, char *passwd, struct authen_data *data, int recurse)
      * If there is no login or pap password for this user, see if there is
      * a global password that can be used.
      */
-    if (!cfg_passwd) {
+    if (cfg_passwd == NULL) {
 	cfg_passwd = cfg_get_global_secret(name, recurse);
     }
 
@@ -135,7 +140,7 @@ verify(char *name, char *passwd, struct authen_data *data, int recurse)
      * matter) but the default authentication = file <file> statement
      * has been issued, attempt to use this password file
      */
-    if (!cfg_passwd) {
+    if (cfg_passwd == NULL) {
 	char *file = cfg_get_authen_default();
 	if (file) {
 	    return(passwd_file_verify(name, passwd, data, file));
@@ -163,7 +168,7 @@ verify(char *name, char *passwd, struct authen_data *data, int recurse)
 #endif
 
     p = tac_find_substring("cleartext ", cfg_passwd);
-    if (p) {
+    if (p != NULL) {
 	if (debug & DEBUG_PASSWD_FLAG)
 	    report(LOG_DEBUG, "verify daemon %s == NAS %s", p, passwd);
 
@@ -204,7 +209,8 @@ verify(char *name, char *passwd, struct authen_data *data, int recurse)
 	return(passwd_file_verify(name, passwd, data, p));
     }
 
-    /* Oops. No idea what kind of password this is. This should never
+    /*
+     * Oops. No idea what kind of password this is. This should never
      * happen as the parser should never create such passwords.
      */
     report(LOG_ERR, "%s: Error cannot identify password type %s for %s",
@@ -344,10 +350,10 @@ etc_passwd_file_verify(char *user, char *supplied_passwd,
 	 * Convert this to ascii so that the traditional tacacs
 	 * password expiration routines work correctly.
 	 */
-
 	if (spwd->sp_expire > 0) {
 	    long secs = spwd->sp_expire * 24 * 60 * 60;
 	    char *p = ctime(&secs);
+
 	    memcpy(buf, p + 4, 7);
 	    memcpy(buf + 7, p + 20, 4);
 	    buf[11] = '\0';
@@ -444,7 +450,7 @@ des_verify(char *users_passwd, char *encrypted_passwd)
 	return(0);
     }
 
-    ep = (char *) crypt(users_passwd, encrypted_passwd);
+    ep = (char *)crypt(users_passwd, encrypted_passwd);
 
     if (debug & DEBUG_PASSWD_FLAG)
 	report(LOG_DEBUG, "%s encrypts to %s", users_passwd, ep);
@@ -462,12 +468,14 @@ des_verify(char *users_passwd, char *encrypted_passwd)
 }
 
 #if HAVE_PAM
+/* pam_conv (PAM conversation) callback */
 static int
 pam_tacacs(int nmsg, const struct pam_message **pmpp, struct pam_response
 	   **prpp, void *appdata_ptr)
 {
     int i;
     struct authen_cont *acp;
+    char *passwd = (char *)appdata_ptr;
     u_char *reply, *rp;
 
     if (debug & DEBUG_PASSWD_FLAG)
@@ -478,43 +486,49 @@ pam_tacacs(int nmsg, const struct pam_message **pmpp, struct pam_response
     if ((*prpp = (struct pam_response *)
 		 tac_malloc(nmsg * sizeof(struct pam_response))) == NULL)
 	return(PAM_BUF_ERR);
-    memset((struct pam_repsonse *) *prpp, 0,
+    memset((struct pam_repsonse *)*prpp, 0,
 	   nmsg * sizeof(struct pam_response));
 
     for (i = 0; i < nmsg; ++i) {
 	switch (pmpp[i]->msg_style) {
 	case PAM_PROMPT_ECHO_OFF:
 	    if (debug & DEBUG_PASSWD_FLAG)
-		report(LOG_ERR, "%s %s: PAM_PROMPT_ECHO_OFF", session.peer,
+		report(LOG_DEBUG, "%s %s: PAM_PROMPT_ECHO_OFF", session.peer,
 		       session.port);
 
-	    send_authen_reply(TAC_PLUS_AUTHEN_STATUS_GETPASS,
-			      (char *)pmpp[i]->msg,
-			      pmpp[i]->msg ? strlen(pmpp[i]->msg) : 0,
-			      NULL, 0, TAC_PLUS_AUTHEN_FLAG_NOECHO);
-	    reply = get_authen_continue();
-	    if (!reply) {
-		/* Typically due to a premature connection close */
-		report(LOG_ERR, "%s %s: Null reply packet, expecting CONTINUE",
-		       session.peer, session.port);
- 		goto fail;
+	    /* pre-supplied password, such as service=PAP, or prompt for it */
+	    if (passwd != NULL && strlen(passwd) > 0) {
+		prpp[i]->resp = tac_strdup(passwd);
+	    } else {
+		send_authen_reply(TAC_PLUS_AUTHEN_STATUS_GETPASS,
+				  (char *)pmpp[i]->msg,
+				  pmpp[i]->msg ? strlen(pmpp[i]->msg) : 0,
+				  NULL, 0, TAC_PLUS_AUTHEN_FLAG_NOECHO);
+		reply = get_authen_continue();
+		if (!reply) {
+		    /* Typically due to a premature connection close */
+		    report(LOG_ERR, "%s %s: Null reply packet, expecting "
+			   "CONTINUE", session.peer, session.port);
+ 		    goto fail;
+		}
+		acp = (struct authen_cont *)(reply + TAC_PLUS_HDR_SIZE);
+
+		rp = reply + TAC_PLUS_HDR_SIZE +
+		     TAC_AUTHEN_CONT_FIXED_FIELDS_SIZE;
+		/*
+		 * A response to our GETDATA/GETPASS request. Create a
+		 * null-terminated string for authen_data.
+		 */
+		prpp[i]->resp = (char *)tac_malloc(acp->user_msg_len + 1);
+		memcpy(prpp[i]->resp, rp, acp->user_msg_len);
+		prpp[i]->resp[acp->user_msg_len] = '\0';
+
+		free(reply);
 	    }
-	    acp = (struct authen_cont *) (reply + TAC_PLUS_HDR_SIZE);
-
-	    rp = reply + TAC_PLUS_HDR_SIZE + TAC_AUTHEN_CONT_FIXED_FIELDS_SIZE;
-	    /*
-	     * A response to our GETDATA/GETPASS request. Create a
-	     * null-terminated string for authen_data.
-	     */
-	    prpp[i]->resp = (char *) tac_malloc(acp->user_msg_len + 1);
-	    memcpy(prpp[i]->resp, rp, acp->user_msg_len);
-	    prpp[i]->resp[acp->user_msg_len] = '\0';
-
-	    free(reply);
 	    break;
 	case PAM_PROMPT_ECHO_ON:
 	    if (debug & DEBUG_PASSWD_FLAG)
-		report(LOG_ERR, "%s %s: PAM_PROMPT_ECHO_ON", session.peer,
+		report(LOG_DEBUG, "%s %s: PAM_PROMPT_ECHO_ON", session.peer,
 		       session.port);
 
 	    send_authen_reply(TAC_PLUS_AUTHEN_STATUS_GETDATA,
@@ -528,14 +542,14 @@ pam_tacacs(int nmsg, const struct pam_message **pmpp, struct pam_response
 		       session.peer, session.port);
  		goto fail;
 	    }
-	    acp = (struct authen_cont *) (reply + TAC_PLUS_HDR_SIZE);
+	    acp = (struct authen_cont *)(reply + TAC_PLUS_HDR_SIZE);
 
 	    rp = reply + TAC_PLUS_HDR_SIZE + TAC_AUTHEN_CONT_FIXED_FIELDS_SIZE;
 	    /*
 	     * A response to our GETDATA/GETPASS request. Create a
 	     * null-terminated string for authen_data.
 	     */
-	    prpp[i]->resp = (char *) tac_malloc(acp->user_msg_len + 1);
+	    prpp[i]->resp = (char *)tac_malloc(acp->user_msg_len + 1);
 	    memcpy(prpp[i]->resp, rp, acp->user_msg_len);
 	    prpp[i]->resp[acp->user_msg_len] = '\0';
 
@@ -584,7 +598,7 @@ pam_verify(char *user, char *passwd)
 {
     int			err;
     int			pam_flag;
-    struct pam_conv	conv = { pam_tacacs, NULL };
+    struct pam_conv	conv = { pam_tacacs, passwd };
     pam_handle_t	*pamh = NULL;
 
     if (debug & DEBUG_PASSWD_FLAG)
